part of disposable;

/// Provides a stand-alone implementation of ensuring a batch of disposable instances are all cleaned-up
class CompositeDisposable implements IFutureDisposable {
  final List<IDisposable> _disposables;
  bool _isDisposed = false;
  bool get isDisposed => _isDisposed;

  CompositeDisposable._(Iterable<IDisposable> this._disposables);

  /// Instantiates a composite which immediately wraps the provided set of disposables
  factory CompositeDisposable.fromDisposables(Iterable<IDisposable> disposables) {
    return new CompositeDisposable._(disposables);
  }

  /// Instantiates a composite which has no implicit set of disposables
  factory CompositeDisposable.empty() {
    return new CompositeDisposable._([]);
  }

  /// Appends a disposable instance into the set controlled by this composite
  void add(IDisposable disposable) {
    if(_isDisposed) { disposable.dispose(); }
    _disposables.add(disposable);
  }

  /// Removes and also disposes the item removed immediately
  void remove(IDisposable disposable) {
    if(!_disposables.contains(disposable)) { return; }
    int index = _disposables.indexOf(disposable);
    _disposables[index].dispose();
    _disposables.removeAt(index);
  }

  /// Checks for the presence of a disposable instance in the composite set
  bool contains(IDisposable disposable) {
    if(_isDisposed) { return false; }
    return _disposables.contains(disposable);
  }

  /// Triggers disposal of all contained elements and removes them from the composite set
  /// Does not mark the composite set as disposed allowing it to be subsequently disposed at a later point
  void clear() {
    _disposables.forEach((_) => _.dispose());
    _disposables.clear();
  }

  /// Triggers disposal of all contained elements and removes them from the composite set
  /// Also marks the composite set as disposed which disables future dispose calls
  void dispose() {
    if(_isDisposed) { return; }
    _isDisposed = true;
    clear();
  }
}